@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::Applicator;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Applicator vocabulary
 
-our $VERSION = '0.555';
+our $VERSION = '0.556';
 
 use 5.020;
 use Moo;
@@ -69,13 +69,9 @@ sub _traverse_keyword_allOf { shift->traverse_array_schemas(@_) }
 
 sub _eval_keyword_allOf ($self, $data, $schema, $state) {
   my @invalid;
-  my @orig_annotations = $state->{annotations}->@*;
-  my @new_annotations;
   foreach my $idx (0 .. $schema->{allOf}->$#*) {
-    my @annotations = @orig_annotations;
     if ($self->eval($data, $schema->{allOf}[$idx], +{ %$state,
-        schema_path => $state->{schema_path}.'/allOf/'.$idx, annotations => \@annotations })) {
-      push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
+        schema_path => $state->{schema_path}.'/allOf/'.$idx })) {
     }
     else {
       push @invalid, $idx;
@@ -83,7 +79,6 @@ sub _eval_keyword_allOf ($self, $data, $schema, $state) {
     }
   }
 
-  push $state->{annotations}->@*, @new_annotations;
   return 1 if @invalid == 0;
 
   my $pl = @invalid > 1;
@@ -111,19 +106,13 @@ sub _traverse_keyword_oneOf { shift->traverse_array_schemas(@_) }
 
 sub _eval_keyword_oneOf ($self, $data, $schema, $state) {
   my (@valid, @errors);
-  my @orig_annotations = $state->{annotations}->@*;
-  my @new_annotations;
   foreach my $idx (0 .. $schema->{oneOf}->$#*) {
-    my @annotations = @orig_annotations;
     next if not $self->eval($data, $schema->{oneOf}[$idx],
-      +{ %$state, errors => \@errors, annotations => \@annotations,
-        schema_path => $state->{schema_path}.'/oneOf/'.$idx });
+      +{ %$state, errors => \@errors, schema_path => $state->{schema_path}.'/oneOf/'.$idx });
     push @valid, $idx;
-    push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
     last if @valid > 1 and $state->{short_circuit};
   }
 
-  push $state->{annotations}->@*, @new_annotations;
   return 1 if @valid == 1;
 
   if (not @valid) {
@@ -172,16 +161,11 @@ sub _eval_keyword_dependentSchemas ($self, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
-  my @orig_annotations = $state->{annotations}->@*;
-  my @new_annotations;
   foreach my $property (sort keys $schema->{dependentSchemas}->%*) {
     next if not exists $data->{$property};
 
-    my @annotations = @orig_annotations;
     if ($self->eval($data, $schema->{dependentSchemas}{$property},
-        +{ %$state, annotations => \@annotations,
-          schema_path => jsonp($state->{schema_path}, 'dependentSchemas', $property) })) {
-      push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
+        +{ %$state, schema_path => jsonp($state->{schema_path}, 'dependentSchemas', $property) })) {
       next;
     }
 
@@ -189,7 +173,6 @@ sub _eval_keyword_dependentSchemas ($self, $data, $schema, $state) {
     last if $state->{short_circuit};
   }
 
-  push $state->{annotations}->@*, @new_annotations;
   return E($state, 'not all dependencies are satisfied') if not $valid;
   return 1;
 }
@@ -222,8 +205,6 @@ sub _eval_keyword_dependencies ($self, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
-  my @orig_annotations = $state->{annotations}->@*;
-  my @new_annotations;
   foreach my $property (sort keys $schema->{dependencies}->%*) {
     next if not exists $data->{$property};
 
@@ -236,11 +217,8 @@ sub _eval_keyword_dependencies ($self, $data, $schema, $state) {
     }
     else {
       # as in dependentSchemas
-      my @annotations = @orig_annotations;
       if ($self->eval($data, $schema->{dependencies}{$property},
-          +{ %$state, annotations => \@annotations,
-            schema_path => jsonp($state->{schema_path}, 'dependencies', $property) })) {
-        push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
+          +{ %$state, schema_path => jsonp($state->{schema_path}, 'dependencies', $property) })) {
         next;
       }
 
@@ -249,7 +227,6 @@ sub _eval_keyword_dependencies ($self, $data, $schema, $state) {
     }
   }
 
-  push $state->{annotations}->@*, @new_annotations;
   return E($state, 'not all dependencies are satisfied') if not $valid;
   return 1;
 }
@@ -288,25 +265,20 @@ sub _eval_keyword__items_array_schemas ($self, $data, $schema, $state) {
   return 1 if not is_type('array', $data);
   return 1 if ($state->{_last_items_index}//-1) == $data->$#*;
 
-  my @orig_annotations = $state->{annotations}->@*;
-  my @new_annotations;
   my $valid = 1;
 
   foreach my $idx (0 .. $data->$#*) {
     last if $idx > $schema->{$state->{keyword}}->$#*;
     $state->{_last_items_index} = $idx;
 
-    my @annotations = @orig_annotations;
     if (is_type('boolean', $schema->{$state->{keyword}}[$idx])) {
       next if $schema->{$state->{keyword}}[$idx];
       $valid = E({ %$state, data_path => $state->{data_path}.'/'.$idx,
         _schema_path_suffix => $idx }, 'item not permitted');
     }
     elsif ($self->eval($data->[$idx], $schema->{$state->{keyword}}[$idx],
-        +{ %$state, annotations => \@annotations,
-          data_path => $state->{data_path}.'/'.$idx,
+        +{ %$state, data_path => $state->{data_path}.'/'.$idx,
           schema_path => $state->{schema_path}.'/'.$state->{keyword}.'/'.$idx })) {
-      push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
       next;
     }
 
@@ -317,7 +289,6 @@ sub _eval_keyword__items_array_schemas ($self, $data, $schema, $state) {
     };
   }
 
-  push $state->{annotations}->@*, @new_annotations;
   A($state, $state->{_last_items_index} == $data->$#* ? true : $state->{_last_items_index});
   return E($state, 'not all items are valid') if not $valid;
   return 1;
@@ -328,8 +299,6 @@ sub _eval_keyword__items_schema ($self, $data, $schema, $state) {
   return 1 if not is_type('array', $data);
   return 1 if ($state->{_last_items_index}//-1) == $data->$#*;
 
-  my @orig_annotations = $state->{annotations}->@*;
-  my @new_annotations;
   my $valid = 1;
 
   foreach my $idx (($state->{_last_items_index}//-1)+1 .. $data->$#*) {
@@ -340,12 +309,9 @@ sub _eval_keyword__items_schema ($self, $data, $schema, $state) {
         exists $schema->{prefixItems} || $state->{keyword} eq 'additionalItems' ? 'additional ' : '');
     }
     else {
-      my @annotations = @orig_annotations;
       if ($self->eval($data->[$idx], $schema->{$state->{keyword}},
-        +{ %$state, annotations => \@annotations,
-          data_path => $state->{data_path}.'/'.$idx,
+        +{ %$state, data_path => $state->{data_path}.'/'.$idx,
           schema_path => $state->{schema_path}.'/'.$state->{keyword} })) {
-        push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
         next;
       }
 
@@ -356,7 +322,6 @@ sub _eval_keyword__items_schema ($self, $data, $schema, $state) {
 
   $state->{_last_items_index} = $data->$#*;
 
-  push $state->{annotations}->@*, @new_annotations;
   A($state, true);
   return E($state, 'subschema is not valid against all %sitems',
     $state->{keyword} eq 'additionalItems' ? 'additional ' : '') if not $valid;
@@ -369,16 +334,13 @@ sub _eval_keyword_contains ($self, $data, $schema, $state) {
   return 1 if not is_type('array', $data);
 
   $state->{_num_contains} = 0;
-  my @orig_annotations = $state->{annotations}->@*;
-  my (@errors, @new_annotations, @valid);
+  my (@errors, @valid);
   foreach my $idx (0 .. $data->$#*) {
-    my @annotations = @orig_annotations;
     if ($self->eval($data->[$idx], $schema->{contains},
-        +{ %$state, errors => \@errors, annotations => \@annotations,
+        +{ %$state, errors => \@errors,
           data_path => $state->{data_path}.'/'.$idx,
           schema_path => $state->{schema_path}.'/contains' })) {
       ++$state->{_num_contains};
-      push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
       push @valid, $idx;
 
       last if $state->{short_circuit}
@@ -394,7 +356,6 @@ sub _eval_keyword_contains ($self, $data, $schema, $state) {
     return E($state, 'subschema is not valid against any item');
   }
 
-  push $state->{annotations}->@*, @new_annotations;
   return $state->{spec_version} =~ /^draft(?:7|2019-09)$/ ? 1
     : A($state, @valid == @$data ? true : \@valid);
 }
@@ -405,8 +366,7 @@ sub _eval_keyword_properties ($self, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
-  my @orig_annotations = $state->{annotations}->@*;
-  my (@properties, @new_annotations);
+  my @properties;
   foreach my $property (sort keys $schema->{properties}->%*) {
     next if not exists $data->{$property};
     push @properties, $property;
@@ -417,12 +377,9 @@ sub _eval_keyword_properties ($self, $data, $schema, $state) {
         _schema_path_suffix => $property }, 'property not permitted');
     }
     else {
-      my @annotations = @orig_annotations;
       if ($self->eval($data->{$property}, $schema->{properties}{$property},
-          +{ %$state, annotations => \@annotations,
-            data_path => jsonp($state->{data_path}, $property),
+          +{ %$state, data_path => jsonp($state->{data_path}, $property),
             schema_path => jsonp($state->{schema_path}, 'properties', $property) })) {
-        push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
         next;
       }
 
@@ -431,7 +388,6 @@ sub _eval_keyword_properties ($self, $data, $schema, $state) {
     last if $state->{short_circuit};
   }
 
-  push $state->{annotations}->@*, @new_annotations;
   A($state, \@properties);
   return E($state, 'not all properties are valid') if not $valid;
   return 1;
@@ -452,8 +408,7 @@ sub _eval_keyword_patternProperties ($self, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
-  my @orig_annotations = $state->{annotations}->@*;
-  my (@properties, @new_annotations);
+  my @properties;
   foreach my $property_pattern (sort keys $schema->{patternProperties}->%*) {
     foreach my $property (sort grep m/$property_pattern/, keys %$data) {
       push @properties, $property;
@@ -463,12 +418,9 @@ sub _eval_keyword_patternProperties ($self, $data, $schema, $state) {
           _schema_path_suffix => $property_pattern }, 'property not permitted');
       }
       else {
-        my @annotations = @orig_annotations;
         if ($self->eval($data->{$property}, $schema->{patternProperties}{$property_pattern},
-            +{ %$state, annotations => \@annotations,
-              data_path => jsonp($state->{data_path}, $property),
+            +{ %$state, data_path => jsonp($state->{data_path}, $property),
               schema_path => jsonp($state->{schema_path}, 'patternProperties', $property_pattern) })) {
-          push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
           next;
         }
 
@@ -478,7 +430,6 @@ sub _eval_keyword_patternProperties ($self, $data, $schema, $state) {
     }
   }
 
-  push $state->{annotations}->@*, @new_annotations;
   A($state, [ uniqstr @properties ]);
   return E($state, 'not all properties are valid') if not $valid;
   return 1;
@@ -490,8 +441,7 @@ sub _eval_keyword_additionalProperties ($self, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
-  my @orig_annotations = $state->{annotations}->@*;
-  my (@properties, @new_annotations);
+  my @properties;
   foreach my $property (sort keys %$data) {
     next if exists $schema->{properties} and exists $schema->{properties}{$property};
     next if exists $schema->{patternProperties}
@@ -504,12 +454,9 @@ sub _eval_keyword_additionalProperties ($self, $data, $schema, $state) {
         'additional property not permitted');
     }
     else {
-      my @annotations = @orig_annotations;
       if ($self->eval($data->{$property}, $schema->{additionalProperties},
-          +{ %$state, annotations => \@annotations,
-            data_path => jsonp($state->{data_path}, $property),
+          +{ %$state, data_path => jsonp($state->{data_path}, $property),
             schema_path => $state->{schema_path}.'/additionalProperties' })) {
-        push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
         next;
       }
 
@@ -518,7 +465,6 @@ sub _eval_keyword_additionalProperties ($self, $data, $schema, $state) {
     last if $state->{short_circuit};
   }
 
-  push $state->{annotations}->@*, @new_annotations;
   A($state, \@properties);
   return E($state, 'not all additional properties are valid') if not $valid;
   return 1;
@@ -530,15 +476,10 @@ sub _eval_keyword_propertyNames ($self, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
-  my @orig_annotations = $state->{annotations}->@*;
-  my @new_annotations;
   foreach my $property (sort keys %$data) {
-    my @annotations = @orig_annotations;
     if ($self->eval($property, $schema->{propertyNames},
-        +{ %$state, annotations => \@annotations,
-          data_path => jsonp($state->{data_path}, $property),
+        +{ %$state, data_path => jsonp($state->{data_path}, $property),
           schema_path => $state->{schema_path}.'/propertyNames' })) {
-      push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
       next;
     }
 
@@ -546,7 +487,6 @@ sub _eval_keyword_propertyNames ($self, $data, $schema, $state) {
     last if $state->{short_circuit};
   }
 
-  push $state->{annotations}->@*, @new_annotations;
   return E($state, 'not all property names are valid') if not $valid;
   return 1;
 }
@@ -565,7 +505,7 @@ JSON::Schema::Modern::Vocabulary::Applicator - Implementation of the JSON Schema
 
 =head1 VERSION
 
-version 0.555
+version 0.556
 
 =head1 DESCRIPTION
 
